@@ -12,10 +12,11 @@
 
 #include "Converter.h"
 
-#include "ConverterCache.h"
 #include "Buffer.h"
 #include "buffer_funcs.h"
 #include "BufferParams.h"
+#include "ConverterCache.h"
+#include "Encoding.h"
 #include "ErrorList.h"
 #include "Format.h"
 #include "Language.h"
@@ -49,7 +50,7 @@ string const token_base("$$b");
 string const token_to("$$o");
 string const token_path("$$p");
 string const token_orig_path("$$r");
-
+string const token_encoding("$$e");
 
 
 string const add_options(string const & command, string const & options)
@@ -107,9 +108,11 @@ void Converter::readFlags()
 		string flag_name, flag_value;
 		flag_list = split(flag_list, flag_value, ',');
 		flag_value = split(flag_value, flag_name, '=');
-		if (flag_name == "latex")
+		if (flag_name == "latex") {
 			latex = true;
-		else if (flag_name == "xml")
+			latex_flavor = flag_value.empty() ?
+				"latex" : flag_value;
+		} else if (flag_name == "xml")
 			xml = true;
 		else if (flag_name == "needaux")
 			need_aux = true;
@@ -185,11 +188,16 @@ void Converters::add(string const & from, string const & to,
 	}
 	converter.readFlags();
 
-	if (converter.latex && (latex_command_.empty() || to == "dvi"))
-		latex_command_ = subst(command, token_from, "");
 	// If we have both latex & pdflatex, we set latex_command to latex.
 	// The latex_command is used to update the .aux file when running
 	// a converter that uses it.
+	if (converter.latex
+	    && (latex_command_.empty() || converter.latex_flavor == "latex"))
+		latex_command_ = subst(command, token_from, "");
+	// Similarly, set xelatex_command to xelatex.
+	if (converter.latex
+	    && (xelatex_command_.empty() || converter.latex_flavor == "xelatex"))
+		xelatex_command_ = subst(command, token_from, "");
 
 	if (it == converterlist_.end()) {
 		converterlist_.push_back(converter);
@@ -254,7 +262,11 @@ OutputParams::FLAVOR Converters::getFlavor(Graph::EdgePath const & path)
 	     cit != path.end(); ++cit) {
 		Converter const & conv = converterlist_[*cit];
 		if (conv.latex)
-			if (contains(conv.to, "pdf"))
+			if (conv.latex_flavor == "xelatex")
+				return OutputParams::XETEX;
+			if (conv.latex_flavor == "lualatex")
+				return OutputParams::LUATEX;
+			if (conv.latex_flavor == "pdflatex")
 				return OutputParams::PDFLATEX;
 		if (conv.xml)
 			return OutputParams::XML;
@@ -282,7 +294,7 @@ bool Converters::convert(Buffer const * buffer,
 			// if no special converter defined, then we take the
 			// default one from ImageMagic.
 			string const from_ext = from_format.empty() ?
-				getExtension(from_file.absFilename()) :
+				getExtension(from_file.absFileName()) :
 				formats.extension(from_format);
 			string const to_ext = formats.extension(to_format);
 			string const command =
@@ -303,6 +315,14 @@ bool Converters::convert(Buffer const * buffer,
 				return true;
 			}
 		}
+
+		// only warn once per session and per file type
+		static std::map<string, string> warned;
+		if (warned.find(from_format) != warned.end() && warned.find(from_format)->second == to_format) {
+			return false;
+		}
+		warned.insert(make_pair(from_format, to_format));
+
 		Alert::error(_("Cannot convert file"),
 			     bformat(_("No information for converting %1$s "
 						    "format files to %2$s.\n"
@@ -315,16 +335,22 @@ bool Converters::convert(Buffer const * buffer,
 	// used anyway.
 	OutputParams runparams(buffer ? &buffer->params().encoding() : 0);
 	runparams.flavor = getFlavor(edgepath);
-	
-	if (buffer)
+
+	if (buffer) {
 		runparams.use_japanese = buffer->bufferFormat() == "platex";
+		runparams.use_indices = buffer->params().use_indices;
+		runparams.bibtex_command = (buffer->params().bibtex_command == "default") ?
+			string() : buffer->params().bibtex_command;
+		runparams.index_command = (buffer->params().index_command == "default") ?
+			string() : buffer->params().index_command;
+	}
 
 	// Some converters (e.g. lilypond) can only output files to the
 	// current directory, so we need to change the current directory.
 	// This has the added benefit that all other files that may be
 	// generated by the converter are deleted when LyX closes and do not
 	// clutter the real working directory.
-	string const path(onlyPath(from_file.absFilename()));
+	string const path(onlyPath(from_file.absFileName()));
 	// Prevent the compiler from optimizing away p
 	FileName pp(path);
 	PathChanger p(pp);
@@ -333,8 +359,8 @@ bool Converters::convert(Buffer const * buffer,
 	errorList.clear();
 
 	bool run_latex = false;
-	string from_base = changeExtension(from_file.absFilename(), "");
-	string to_base = changeExtension(to_file.absFilename(), "");
+	string from_base = changeExtension(from_file.absFileName(), "");
+	string to_base = changeExtension(to_file.absFileName(), "");
 	FileName infile;
 	FileName outfile = from_file;
 	for (Graph::EdgePath::const_iterator cit = edgepath.begin();
@@ -347,11 +373,11 @@ bool Converters::convert(Buffer const * buffer,
 		}
 		infile = outfile;
 		outfile = FileName(conv.result_dir.empty()
-			? changeExtension(from_file.absFilename(), conv.To->extension())
+			? changeExtension(from_file.absFileName(), conv.To->extension())
 			: addName(subst(conv.result_dir,
 					token_base, from_base),
 				  subst(conv.result_file,
-					token_base, onlyFilename(from_base))));
+					token_base, onlyFileName(from_base))));
 
 		// if input and output files are equal, we use a
 		// temporary file as intermediary (JMarc)
@@ -362,7 +388,7 @@ bool Converters::convert(Buffer const * buffer,
 			if (buffer)
 				outfile = FileName(addName(buffer->temppath(), "tmpfile.out"));
 			else
-				outfile = FileName(addName(package().temp_dir().absFilename(),
+				outfile = FileName(addName(package().temp_dir().absFileName(),
 						   "tmpfile.out"));
 		}
 
@@ -375,23 +401,27 @@ bool Converters::convert(Buffer const * buffer,
 		} else {
 			if (conv.need_aux && !run_latex
 			    && !latex_command_.empty()) {
-				LYXERR(Debug::FILES, "Running " << latex_command_
+				string const command = (buffer && buffer->params().useNonTeXFonts) ?
+					xelatex_command_ : latex_command_;
+				LYXERR(Debug::FILES, "Running " << command
 					<< " to update aux file");
-				runLaTeX(*buffer, latex_command_, runparams, errorList);
+				if (!runLaTeX(*buffer, command, runparams, errorList))
+					return false;
 			}
 
 			// FIXME UNICODE
-			string const infile2 = 
-				to_utf8(makeRelPath(from_utf8(infile.absFilename()), from_utf8(path)));
-			string const outfile2 = 
-				to_utf8(makeRelPath(from_utf8(outfile.absFilename()), from_utf8(path)));
+			string const infile2 =
+				to_utf8(makeRelPath(from_utf8(infile.absFileName()), from_utf8(path)));
+			string const outfile2 =
+				to_utf8(makeRelPath(from_utf8(outfile.absFileName()), from_utf8(path)));
 
 			string command = conv.command;
 			command = subst(command, token_from, quoteName(infile2));
 			command = subst(command, token_base, quoteName(from_base));
 			command = subst(command, token_to, quoteName(outfile2));
-			command = subst(command, token_path, quoteName(infile.onlyPath().absFilename()));
-			command = subst(command, token_orig_path, quoteName(orig_from.onlyPath().absFilename()));
+			command = subst(command, token_path, quoteName(onlyPath(infile.absFileName())));
+			command = subst(command, token_orig_path, quoteName(onlyPath(orig_from.absFileName())));
+			command = subst(command, token_encoding, buffer ? buffer->params().encoding().iconvName() : string());
 			command = libScriptSearch(command);
 
 			if (!conv.parselog.empty())
@@ -452,8 +482,8 @@ bool Converters::convert(Buffer const * buffer,
 // FIXME: this should go out of here. For example, here we cannot say if
 // it is a document (.lyx) or something else. Same goes for elsewhere.
 					Alert::error(_("Cannot convert file"),
-						bformat(_("An error occurred whilst running %1$s"),
-						from_utf8(command.substr(0, 50))));
+						bformat(_("An error occurred while running:\n%1$s"),
+						wrapParas(from_utf8(command))));
 				}
 				return false;
 			}
@@ -496,16 +526,16 @@ bool Converters::move(string const & fmt,
 		return true;
 
 	bool no_errors = true;
-	string const path = onlyPath(from.absFilename());
-	string const base = onlyFilename(removeExtension(from.absFilename()));
-	string const to_base = removeExtension(to.absFilename());
-	string const to_extension = getExtension(to.absFilename());
+	string const path = onlyPath(from.absFileName());
+	string const base = onlyFileName(removeExtension(from.absFileName()));
+	string const to_base = removeExtension(to.absFileName());
+	string const to_extension = getExtension(to.absFileName());
 
-	support::FileNameList const files = FileName(path).dirList(getExtension(from.absFilename()));
+	support::FileNameList const files = FileName(path).dirList(getExtension(from.absFileName()));
 	for (support::FileNameList::const_iterator it = files.begin();
 	     it != files.end(); ++it) {
-		string const from2 = it->absFilename();
-		string const file2 = onlyFilename(from2);
+		string const from2 = it->absFileName();
+		string const file2 = onlyFileName(from2);
 		if (prefixIs(file2, base)) {
 			string const to2 = changeExtension(
 				to_base + file2.substr(base.length()),
@@ -592,13 +622,13 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 		buffer.bufferErrors(terr, errorList);
 
 	// check return value from latex.run().
-	if ((result & LaTeX::NO_LOGFILE)) {
+	if ((result & LaTeX::NO_LOGFILE) && !buffer.isClone()) {
 		docstring const str =
 			bformat(_("LaTeX did not run successfully. "
 					       "Additionally, LyX could not locate "
 					       "the LaTeX log %1$s."), from_utf8(name));
 		Alert::error(_("LaTeX failed"), str);
-	} else if (result & LaTeX::NO_OUTPUT) {
+	} else if ((result & LaTeX::NO_OUTPUT) && !buffer.isClone()) {
 		Alert::warning(_("Output is empty"),
 			       _("An empty output file was generated."));
 	}
@@ -619,13 +649,17 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 
 void Converters::buildGraph()
 {
+	// clear graph's data structures
 	G_.init(formats.size());
-	ConverterList::iterator beg = converterlist_.begin();
+	// each of the converters knows how to convert one format to another
+	// so, for each of them, we create an arrow on the graph, going from 
+	// the one to the other
+	ConverterList::iterator it = converterlist_.begin();
 	ConverterList::iterator const end = converterlist_.end();
-	for (ConverterList::iterator it = beg; it != end ; ++it) {
-		int const s = formats.getNumber(it->from);
-		int const t = formats.getNumber(it->to);
-		G_.addEdge(s,t);
+	for (; it != end ; ++it) {
+		int const from = formats.getNumber(it->from);
+		int const to   = formats.getNumber(it->to);
+		G_.addEdge(from, to);
 	}
 }
 
@@ -657,12 +691,20 @@ Converters::getReachableTo(string const & target, bool const clear_visited)
 
 vector<Format const *> const
 Converters::getReachable(string const & from, bool const only_viewable,
-			 bool const clear_visited)
+			 bool const clear_visited, set<string> const & excludes)
 {
+	set<int> excluded_numbers;;
+
+	set<string>::const_iterator sit = excludes.begin();
+	set<string>::const_iterator const end = excludes.end();
+	for (; sit != end; ++sit)
+		excluded_numbers.insert(formats.getNumber(*sit));
+
 	vector<int> const & reachables =
 		G_.getReachable(formats.getNumber(from),
 				only_viewable,
-				clear_visited);
+				clear_visited,
+				excluded_numbers);
 
 	return intToFormat(reachables);
 }
@@ -686,8 +728,9 @@ vector<Format const *> Converters::importableFormats()
 {
 	vector<string> l = loaders();
 	vector<Format const *> result = getReachableTo(l[0], true);
-	for (vector<string>::const_iterator it = l.begin() + 1;
-	     it != l.end(); ++it) {
+	vector<string>::const_iterator it = l.begin() + 1;
+	vector<string>::const_iterator en = l.end();
+	for (; it != en; ++it) {
 		vector<Format const *> r = getReachableTo(*it, false);
 		result.insert(result.end(), r.begin(), r.end());
 	}
@@ -695,6 +738,21 @@ vector<Format const *> Converters::importableFormats()
 }
 
 
+vector<Format const *> Converters::exportableFormats(bool only_viewable)
+{
+	vector<string> s = savers();
+	vector<Format const *> result = getReachable(s[0], only_viewable, true);
+	vector<string>::const_iterator it = s.begin() + 1;
+	vector<string>::const_iterator en = s.end();
+	for (; it != en; ++it) {
+		vector<Format const *> r =
+			getReachable(*it, only_viewable, false);
+		result.insert(result.end(), r.begin(), r.end());
+	}
+	return result;
+}
+
+
 vector<string> Converters::loaders() const
 {
 	vector<string> v;
@@ -705,4 +763,21 @@ vector<string> Converters::loaders() const
 }
 
 
+vector<string> Converters::savers() const
+{
+	vector<string> v;
+	v.push_back("docbook");
+	v.push_back("latex");
+	v.push_back("literate");
+	v.push_back("luatex");
+	v.push_back("lyx");
+	v.push_back("xhtml");
+	v.push_back("pdflatex");
+	v.push_back("platex");
+	v.push_back("text");
+	v.push_back("xetex");
+	return v;
+}
+
+
 } // namespace lyx
