@@ -19,24 +19,36 @@
 #include "TocModel.h"
 
 #include "Buffer.h"
+#include "BufferView.h"
+#include "CutAndPaste.h"
 #include "FuncRequest.h"
-#include "LyXFunc.h"
+#include "FuncStatus.h"
+#include "LyX.h"
+#include "Menus.h"
+#include "TocBackend.h"
+
+#include "insets/InsetCommand.h"
+#include "insets/InsetRef.h"
 
 #include "support/debug.h"
 #include "support/lassert.h"
 
 #include <QHeaderView>
+#include <QMenu>
 #include <QTimer>
 
 #include <vector>
 
+#define DELAY_UPDATE_VIEW
+
 using namespace std;
 
 namespace lyx {
 namespace frontend {
 
 TocWidget::TocWidget(GuiView & gui_view, QWidget * parent)
-	: QWidget(parent), depth_(0), persistent_(false), gui_view_(gui_view)
+	: QWidget(parent), depth_(0), persistent_(false), gui_view_(gui_view), update_delay_(0)
+
 {
 	setupUi(this);
 
@@ -59,6 +71,7 @@ TocWidget::TocWidget(GuiView & gui_view, QWidget * parent)
 
 	// Only one item selected at a time.
 	tocTV->setSelectionMode(QAbstractItemView::SingleSelection);
+	setFocusProxy(tocTV);
 
 	// The toc types combo won't change its model.
 	typeCO->setModel(gui_view_.tocModels().nameModel());
@@ -67,20 +80,154 @@ TocWidget::TocWidget(GuiView & gui_view, QWidget * parent)
 	// Buffer.
 	enableControls(false);
 
+	// make us responsible for the context menu of the tabbar
+	setContextMenuPolicy(Qt::CustomContextMenu);
+	connect(this, SIGNAL(customContextMenuRequested(const QPoint &)),
+		this, SLOT(showContextMenu(const QPoint &)));
+	connect(tocTV, SIGNAL(customContextMenuRequested(const QPoint &)),
+		this, SLOT(showContextMenu(const QPoint &)));
+	connect(filterLE, SIGNAL(textEdited(QString)), 
+		this, SLOT(filterContents()));
+
 	init(QString());
 }
 
 
+void TocWidget::showContextMenu(const QPoint & pos)
+{
+	std::string name = "context-toc-" + fromqstr(current_type_);
+	QMenu * menu = guiApp->menus().menu(toqstr(name), gui_view_);
+	if (!menu)
+		return;	
+	menu->exec(mapToGlobal(pos));
+}
+
+
+Inset * TocWidget::itemInset() const
+{
+	QModelIndex const & index = tocTV->currentIndex();
+	TocItem const & item =
+		gui_view_.tocModels().currentItem(current_type_, index);
+	DocIterator const & dit = item.dit();
+	
+	Inset * inset = 0;
+	if (current_type_ == "label" 
+		  || current_type_ == "graphics"
+		  || current_type_ == "citation"
+		  || current_type_ == "child")
+		inset = dit.nextInset();
+
+	else if (current_type_ == "branch"
+			 || current_type_ == "index"
+			 || current_type_ == "change")
+		inset = &dit.inset();
+
+	else if (current_type_ == "table" 
+		     || current_type_ == "listing"
+		     || current_type_ == "figure") {
+		DocIterator tmp_dit(dit);
+		tmp_dit.pop_back();
+		inset = &tmp_dit.inset();
+	}
+	return inset;
+}
+
+
+bool TocWidget::getStatus(Cursor & cur, FuncRequest const & cmd,
+	FuncStatus & status) const
+{
+	Inset * inset = itemInset();
+	FuncRequest tmpcmd(cmd);
+
+	QModelIndex const & index = tocTV->currentIndex();
+	TocItem const & item =
+		gui_view_.tocModels().currentItem(current_type_, index);
+
+	switch (cmd.action())
+	{
+	case LFUN_CHANGE_ACCEPT:
+	case LFUN_CHANGE_REJECT:
+	case LFUN_OUTLINE_UP:
+	case LFUN_OUTLINE_DOWN:
+	case LFUN_OUTLINE_IN:
+	case LFUN_OUTLINE_OUT:
+	case LFUN_SECTION_SELECT:
+		status.setEnabled(true);
+		return true;
+
+	case LFUN_LABEL_COPY_AS_REF: {
+		// For labels in math, we need to supply the label as a string
+		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REF, item.asString());
+		if (inset)
+			return inset->getStatus(cur, label_copy, status);
+	}
+
+	default:
+		if (inset)
+			return inset->getStatus(cur, tmpcmd, status);
+	}
+
+	return false;
+}
+
+
+void TocWidget::doDispatch(Cursor & cur, FuncRequest const & cmd)
+{
+	Inset * inset = itemInset();
+	FuncRequest tmpcmd(cmd);
+
+	QModelIndex const & index = tocTV->currentIndex();
+	TocItem const & item =
+		gui_view_.tocModels().currentItem(current_type_, index);
+
+	// Start an undo group.
+	cur.beginUndoGroup();
+
+	switch (cmd.action())
+	{
+	case LFUN_CHANGE_ACCEPT:
+	case LFUN_CHANGE_REJECT:
+	case LFUN_SECTION_SELECT:
+		dispatch(item.action());
+		cur.dispatch(tmpcmd);
+		break;
+
+	case LFUN_LABEL_COPY_AS_REF: {
+		// For labels in math, we need to supply the label as a string
+		FuncRequest label_copy(LFUN_LABEL_COPY_AS_REF, item.asString());
+		if (inset)
+			inset->dispatch(cur, label_copy);
+		break;
+	}
+	
+	case LFUN_OUTLINE_UP:
+	case LFUN_OUTLINE_DOWN:
+	case LFUN_OUTLINE_IN:
+	case LFUN_OUTLINE_OUT:
+		outline(cmd.action());
+		break;
+
+	default:
+		if (inset)
+			inset->dispatch(cur, tmpcmd);
+	}
+	cur.endUndoGroup();
+}
+
+
 void TocWidget::on_tocTV_activated(QModelIndex const & index)
 {
 	goTo(index);
 }
 
 
-void TocWidget::on_tocTV_clicked(QModelIndex const & index)
+void TocWidget::on_tocTV_pressed(QModelIndex const & index)
 {
-	goTo(index);
-	gui_view_.setFocus();
+	Qt::MouseButtons const button = QApplication::mouseButtons();
+	if (button & Qt::LeftButton) {
+		goTo(index);
+		gui_view_.setFocus();
+	}
 }
 
 
@@ -105,7 +252,7 @@ void TocWidget::on_updateTB_clicked()
 void TocWidget::on_sortCB_stateChanged(int state)
 {
 	gui_view_.tocModels().sort(current_type_, state == Qt::Checked);
-	updateView();
+	updateViewForce();
 }
 
 
@@ -152,7 +299,7 @@ void TocWidget::setTreeDepth(int depth)
 		tocTV->expandToDepth(depth - 1);
 #else
 	// expanding and then collapsing is probably better,
-	// but my qt 4.1.2 doesn't have expandAll().
+	// but my qt 4.1.2 doesn't have expandAll()..
 	//tocTV->expandAll();
 	QModelIndexList indices = tocTV->model()->match(
 		tocTV->model()->index(0, 0),
@@ -173,20 +320,20 @@ void TocWidget::on_typeCO_currentIndexChanged(int index)
 	if (index == -1)
 		return;
 	current_type_ = typeCO->itemData(index).toString();
-	updateView();
-	gui_view_.setFocus();
+	updateViewForce();
+	if (typeCO->hasFocus())
+		gui_view_.setFocus();
 }
 
 
-void TocWidget::outline(int func_code)
+void TocWidget::outline(FuncCode func_code)
 {
-	enableControls(false);
 	QModelIndexList const & list = tocTV->selectionModel()->selectedIndexes();
 	if (list.isEmpty())
 		return;
 	enableControls(false);
 	goTo(list[0]);
-	dispatch(FuncRequest(static_cast<FuncCode>(func_code)));
+	dispatch(FuncRequest(func_code));
 	enableControls(true);
 	gui_view_.setFocus();
 }
@@ -229,18 +376,11 @@ void TocWidget::select(QModelIndex const & index)
 }
 
 
-/// Test whether outlining operation is possible
-static bool canOutline(QString const & type)
-{
-	return type == "tableofcontents";
-}
-
-
 void TocWidget::enableControls(bool enable)
 {
 	updateTB->setEnabled(enable);
 
-	if (!canOutline(current_type_))
+	if (!canOutline())
 		enable = false;
 
 	moveUpTB->setEnabled(enable);
@@ -250,46 +390,45 @@ void TocWidget::enableControls(bool enable)
 }
 
 
-/// Test whether synchronized navigation is possible
-static bool canNavigate(QString const & type)
-{
-	// It is not possible to have synchronous navigation in a correct
-	// and efficient way with the label and change type because Toc::item()
-	// does a linear search. Even when fixed, it might even not be desirable
-	// to do so if we want to support drag&drop of labels and references.
-	return type != "label" && type != "change";
-}
-
-
-/// Test whether sorting is possible
-static bool isSortable(QString const & type)
-{
-	return type != "tableofcontents";
-}
-
-
 void TocWidget::updateView()
 {
-	if (!gui_view_.view()) {
-		enableControls(false);
-		typeCO->setEnabled(false);
+// Enable if you dont want the delaying business, cf #7138.
+#ifndef DELAY_UPDATE_VIEW
+	updateViewForce();
+	return;
+#endif
+	// already scheduled?
+	if (update_delay_ == -1)
+		return;
+	QTimer::singleShot(update_delay_, this, SLOT(updateViewForce()));
+	// Subtler optimization for having the delay more UI invisible.
+	// We trigger update immediately for sparse editation actions,
+	// i.e. there was no editation/cursor movement in last 2 sec.
+	// At worst there will be +1 redraw after 2s in a such "calm" mode.
+	if (update_delay_ != 0)
+		updateViewForce();
+	update_delay_ = -1;
+}
+
+void TocWidget::updateViewForce()
+{
+	update_delay_ = 2000;
+	if (!gui_view_.documentBufferView()) {
 		tocTV->setModel(0);
-		tocTV->setEnabled(false);
 		depthSL->setMaximum(0);
 		depthSL->setValue(0);
-		persistentCB->setEnabled(false);
-		sortCB->setEnabled(false);
-		depthSL->setEnabled(false);
+		setEnabled(false);
 		return;
 	}
-	sortCB->setEnabled(isSortable(current_type_));
-	depthSL->setEnabled(true);
-	typeCO->setEnabled(true);
+	setEnabled(true);
+	bool const is_sortable = isSortable();
+	sortCB->setEnabled(is_sortable);
+	bool focus_ = tocTV->hasFocus();
 	tocTV->setEnabled(false);
 	tocTV->setUpdatesEnabled(false);
 
-	QAbstractItemModel * toc_model =
-		gui_view_.tocModels().model(current_type_);
+	QAbstractItemModel * toc_model = 
+			gui_view_.tocModels().model(current_type_);
 	if (tocTV->model() != toc_model) {
 		tocTV->setModel(toc_model);
 		tocTV->setEditTriggers(QAbstractItemView::NoEditTriggers);
@@ -298,15 +437,15 @@ void TocWidget::updateView()
 	}
 
 	sortCB->blockSignals(true);
-	sortCB->setChecked(isSortable(current_type_)
+	sortCB->setChecked(is_sortable
 		&& gui_view_.tocModels().isSorted(current_type_));
 	sortCB->blockSignals(false);
 
-	bool const can_navigate_ = canNavigate(current_type_);
+	bool const can_navigate_ = canNavigate();
 	persistentCB->setEnabled(can_navigate_);
 
 	bool controls_enabled = toc_model && toc_model->rowCount() > 0
-		&& !gui_view_.buffer()->isReadonly();
+		&& !gui_view_.documentBufferView()->buffer().isReadonly();
 	enableControls(controls_enabled);
 
 	depthSL->setMaximum(gui_view_.tocModels().depth(current_type_));
@@ -317,8 +456,40 @@ void TocWidget::updateView()
 		persistentCB->setChecked(persistent_);
 		select(gui_view_.tocModels().currentIndex(current_type_));
 	}
+	filterContents();
 	tocTV->setEnabled(true);
 	tocTV->setUpdatesEnabled(true);
+	if (focus_)
+		tocTV->setFocus();
+}
+
+
+void TocWidget::filterContents()
+{
+	if (!tocTV->model())
+		return;
+
+	QModelIndexList indices = tocTV->model()->match(
+		tocTV->model()->index(0, 0),
+		Qt::DisplayRole, "*", -1,
+		Qt::MatchFlags(Qt::MatchWildcard|Qt::MatchRecursive));
+
+	int size = indices.size();
+	for (int i = 0; i < size; i++) {
+		QModelIndex index = indices[i];
+		bool const matches =
+			index.data().toString().contains(
+				filterLE->text(), Qt::CaseInsensitive);
+		tocTV->setRowHidden(index.row(), index.parent(), !matches);
+	}
+	// recursively unhide parents of unhidden children 
+	for (int i = size - 1; i >= 0; i--) {
+		QModelIndex index = indices[i];
+		if (!tocTV->isRowHidden(index.row(), index.parent())
+		    && index.parent() != QModelIndex())
+			tocTV->setRowHidden(index.parent().row(),
+					    index.parent().parent(), false);
+	}
 }
 
 
@@ -359,9 +530,12 @@ void TocWidget::init(QString const & str)
 	typeCO->blockSignals(true);
 	typeCO->setCurrentIndex(new_index);
 	typeCO->blockSignals(false);
+
+	// no delay when the whole outliner is reseted.
+	update_delay_ = 0;
 }
 
 } // namespace frontend
 } // namespace lyx
 
-#include "TocWidget_moc.cpp"
+#include "moc_TocWidget.cpp"
