@@ -24,6 +24,7 @@
 #include "support/lstrings.h"
 
 #include <QLineEdit>
+#include <QLocale>
 #include <QWidget>
 
 using namespace std;
@@ -39,10 +40,13 @@ LengthValidator::LengthValidator(QWidget * parent)
 
 QValidator::State LengthValidator::validate(QString & qtext, int &) const
 {
-	string const text = fromqstr(qtext);
-	if (text.empty() || support::isStrDbl(text))
+	bool ok;
+	qtext.trimmed().toDouble(&ok);
+	if (qtext.isEmpty() || ok)
 		return QValidator::Acceptable;
 
+	string const text = fromqstr(qtext);
+
 	if (glue_length_) {
 		GlueLength gl;
 		return (isValidGlueLength(text, &gl)) ?
@@ -93,29 +97,31 @@ LengthValidator * unsignedGlueLengthValidator(QLineEdit * ed)
 }
 
 
-LengthAutoValidator::LengthAutoValidator(QWidget * parent)
-	: LengthValidator(parent)
+LengthAutoValidator::LengthAutoValidator(QWidget * parent, QString const autotext)
+	: LengthValidator(parent),
+	  autotext_(autotext)
 {}
 
 
 QValidator::State LengthAutoValidator::validate(QString & qtext, int & dummy) const
 {
-	if (qtext == "auto")
+	if (qtext == autotext_)
 		return QValidator::Acceptable;
 	return LengthValidator::validate(qtext, dummy);
 }
 
 
-LengthAutoValidator * unsignedLengthAutoValidator(QLineEdit * ed)
+LengthAutoValidator * unsignedLengthAutoValidator(QLineEdit * ed, QString const autotext)
 {
-	LengthAutoValidator * v = new LengthAutoValidator(ed);
+	LengthAutoValidator * v = new LengthAutoValidator(ed, autotext);
 	v->setBottom(Length());
 	return v;
 }
 
 
-DoubleAutoValidator::DoubleAutoValidator(QWidget * parent)
-	: QDoubleValidator(parent)
+DoubleAutoValidator::DoubleAutoValidator(QWidget * parent, QString const autotext)
+	: QDoubleValidator(parent),
+	  autotext_(autotext)
 {}
 
 
@@ -126,7 +132,7 @@ DoubleAutoValidator::DoubleAutoValidator(double bottom,
 
 
 QValidator::State DoubleAutoValidator::validate(QString & input, int & pos) const {
-	if (input == "auto")
+	if (input == autotext_)
 		return QValidator::Acceptable;
 	return QDoubleValidator::validate(input, pos);
 }
@@ -211,4 +217,4 @@ PathValidator * getPathValidator(QLineEdit * ed)
 } // namespace frontend
 } // namespace lyx
 
-#include "Validator_moc.cpp"
+#include "moc_Validator.cpp"
