@@ -12,7 +12,7 @@
 
 #include "InsetText.h"
 
-#include "insets/InsetOptArg.h"
+#include "insets/InsetArgument.h"
 
 #include "buffer_funcs.h"
 #include "Buffer.h"
@@ -26,17 +26,20 @@
 #include "ErrorList.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
+#include "InsetCaption.h"
 #include "InsetList.h"
 #include "Intl.h"
+#include "Language.h"
+#include "LaTeXFeatures.h"
 #include "Lexer.h"
 #include "lyxfind.h"
 #include "LyXRC.h"
 #include "MetricsInfo.h"
 #include "output_docbook.h"
 #include "output_latex.h"
+#include "output_xhtml.h"
 #include "OutputParams.h"
 #include "output_plaintext.h"
-#include "paragraph_funcs.h"
 #include "Paragraph.h"
 #include "ParagraphParameters.h"
 #include "ParIterator.h"
@@ -55,14 +58,15 @@
 #include "support/gettext.h"
 #include "support/lstrings.h"
 
-#include <boost/bind.hpp>
+#include "support/bind.h"
 #include "support/lassert.h"
 
+#include <algorithm>
+
+
 using namespace std;
 using namespace lyx::support;
 
-using boost::bind;
-using boost::ref;
 
 namespace lyx {
 
@@ -71,22 +75,18 @@ using graphics::PreviewLoader;
 
 /////////////////////////////////////////////////////////////////////
 
-InsetText::InsetText(Buffer const & buf)
-	: drawFrame_(false), frame_color_(Color_insetframe)
+InsetText::InsetText(Buffer * buf, UsePlain type)
+	: Inset(buf), drawFrame_(false), frame_color_(Color_insetframe),
+	text_(this, type == DefaultLayout)
 {
-	setBuffer(const_cast<Buffer &>(buf));
-	initParagraphs();
 }
 
 
 InsetText::InsetText(InsetText const & in)
-	: Inset(in), text_()
+	: Inset(in), text_(this, in.text_)
 {
-	text_.autoBreakRows_ = in.text_.autoBreakRows_;
 	drawFrame_ = in.drawFrame_;
 	frame_color_ = in.frame_color_;
-	text_.paragraphs() = in.text_.paragraphs();
-	setParagraphOwner();
 }
 
 
@@ -99,20 +99,23 @@ void InsetText::setBuffer(Buffer & buf)
 }
 
 
-void InsetText::initParagraphs()
+void InsetText::setMacrocontextPositionRecursive(DocIterator const & pos)
 {
-	LASSERT(paragraphs().empty(), /**/);
-	paragraphs().push_back(Paragraph());
-	Paragraph & ourpar = paragraphs().back();
-	ourpar.setInsetOwner(this);
-	ourpar.setDefaultLayout(buffer_->params().documentClass());
-}
+	text_.setMacrocontextPosition(pos);
 
-
-void InsetText::setParagraphOwner()
-{
-	for_each(paragraphs().begin(), paragraphs().end(),
-		 bind(&Paragraph::setInsetOwner, _1, this));
+	ParagraphList::const_iterator pit = paragraphs().begin();
+	ParagraphList::const_iterator pend = paragraphs().end();
+	for (; pit != pend; ++pit) {
+		InsetList::const_iterator iit = pit->insetList().begin();
+		InsetList::const_iterator end = pit->insetList().end();
+		for (; iit != end; ++iit) {
+			if (InsetText * txt = iit->inset->asInsetText()) {
+				DocIterator ppos(pos);
+				ppos.push_back(CursorSlice(*txt));
+				iit->inset->asInsetText()->setMacrocontextPositionRecursive(ppos);
+			}
+		}
+	}
 }
 
 
@@ -145,7 +148,7 @@ Dimension const InsetText::dimension(BufferView const & bv) const
 void InsetText::write(ostream & os) const
 {
 	os << "Text\n";
-	text_.write(buffer(), os);
+	text_.write(os);
 }
 
 
@@ -158,7 +161,7 @@ void InsetText::read(Lexer & lex)
 	paragraphs().clear();
 	ErrorList errorList;
 	lex.setContext("InsetText::read");
-	bool res = text_.read(buffer(), lex, errorList, this);
+	bool res = text_.read(lex, errorList, this);
 
 	if (!res)
 		lex.printError("Missing \\end_inset at this point. ");
@@ -167,6 +170,14 @@ void InsetText::read(Lexer & lex)
 	// ensure we have at least one paragraph.
 	if (paragraphs().empty())
 		paragraphs().push_back(oldpar);
+	// Force default font, if so requested
+	// This avoids paragraphs in buffer language that would have a
+	// foreign language after a document language change, and it ensures
+	// that all new text in ERT and similar gets the "latex" language,
+	// since new text inherits the language from the last position of the
+	// existing text.  As a side effect this makes us also robust against
+	// bugs in LyX that might lead to font changes in ERT in .lyx files.
+	fixParagraphsFont();
 }
 
 
@@ -222,12 +233,6 @@ void InsetText::draw(PainterInfo & pi, int x, int y) const
 }
 
 
-docstring InsetText::editMessage() const
-{
-	return _("Opened Text Inset");
-}
-
-
 void InsetText::edit(Cursor & cur, bool front, EntryDirection entry_from)
 {
 	pit_type const pit = front ? 0 : paragraphs().size() - 1;
@@ -257,32 +262,100 @@ Inset * InsetText::editXY(Cursor & cur, int x, int y)
 void InsetText::doDispatch(Cursor & cur, FuncRequest & cmd)
 {
 	LYXERR(Debug::ACTION, "InsetText::doDispatch()"
-		<< " [ cmd.action = " << cmd.action << ']');
-	text_.dispatch(cur, cmd);
+		<< " [ cmd.action() = " << cmd.action() << ']');
+
+	if (getLayout().isPassThru()) {
+		// Force any new text to latex_language FIXME: This
+		// should only be necessary in constructor, but new
+		// paragraphs that are created by pressing enter at
+		// the start of an existing paragraph get the buffer
+		// language and not latex_language, so we take this
+		// brute force approach.
+		cur.current_font.setLanguage(latex_language);
+		cur.real_current_font.setLanguage(latex_language);
+	}
+
+	switch (cmd.action()) {
+	case LFUN_PASTE:
+	case LFUN_CLIPBOARD_PASTE:
+	case LFUN_SELECTION_PASTE:
+	case LFUN_PRIMARY_SELECTION_PASTE:
+		text_.dispatch(cur, cmd);
+		// If we we can only store plain text, we must reset all
+		// attributes.
+		// FIXME: Change only the pasted paragraphs
+		fixParagraphsFont();
+		break;
+
+	case LFUN_INSET_DISSOLVE: {
+		bool const main_inset = &buffer().inset() == this;
+		bool const target_inset = cmd.argument().empty() 
+			|| cmd.getArg(0) == insetName(lyxCode());
+		bool const one_cell = nargs() == 1;
+
+		if (!main_inset && target_inset && one_cell) {
+			// Text::dissolveInset assumes that the cursor
+			// is inside the Inset.
+			if (&cur.inset() != this)
+				cur.pushBackward(*this);
+			cur.beginUndoGroup();
+			text_.dispatch(cur, cmd);
+			cur.endUndoGroup();
+		} else
+			cur.undispatched();
+		break;
+	}
+
+	default:
+		text_.dispatch(cur, cmd);
+	}
+	
+	if (!cur.result().dispatched())
+		Inset::doDispatch(cur, cmd);
 }
 
 
 bool InsetText::getStatus(Cursor & cur, FuncRequest const & cmd,
 	FuncStatus & status) const
 {
-	switch (cmd.action) {
-	case LFUN_LAYOUT:
-		status.setEnabled(!forcePlainLayout());
-		return true;
+	switch (cmd.action()) {
+	case LFUN_INSET_DISSOLVE: {
+		bool const main_inset = &buffer().inset() == this;
+		bool const target_inset = cmd.argument().empty() 
+			|| cmd.getArg(0) == insetName(lyxCode());
+		bool const one_cell = nargs() == 1;
+
+		if (target_inset)
+			status.setEnabled(!main_inset && one_cell);
+		return target_inset;
+	}
 
-	case LFUN_LAYOUT_PARAGRAPH:
-	case LFUN_PARAGRAPH_PARAMS:
-	case LFUN_PARAGRAPH_PARAMS_APPLY:
-	case LFUN_PARAGRAPH_SPACING:
-	case LFUN_PARAGRAPH_UPDATE:
-		status.setEnabled(allowParagraphCustomization());
-		return true;
 	default:
 		// Dispatch only to text_ if the cursor is inside
 		// the text_. It is not for context menus (bug 5797).
+		bool ret = false;
 		if (cur.text() == &text_)
-			return text_.getStatus(cur, cmd, status);
-		return false;
+			ret = text_.getStatus(cur, cmd, status);
+		
+		if (!ret)
+			ret = Inset::getStatus(cur, cmd, status);
+		return ret;
+	}
+}
+
+
+void InsetText::fixParagraphsFont()
+{
+	Font font(inherit_font, buffer().params().language);
+	font.setLanguage(latex_language);
+	ParagraphList::iterator par = paragraphs().begin();
+	ParagraphList::iterator const end = paragraphs().end();
+	while (par != end) {
+		if (par->isPassThru())
+			par->resetFonts(font);
+		if (!par->allowParagraphCustomization())
+			par->params().clear();
+		++par;
 	}
 }
 
@@ -297,23 +370,81 @@ void InsetText::setChange(Change const & change)
 }
 
 
-void InsetText::acceptChanges(BufferParams const & bparams)
+void InsetText::acceptChanges()
 {
-	text_.acceptChanges(bparams);
+	text_.acceptChanges();
 }
 
 
-void InsetText::rejectChanges(BufferParams const & bparams)
+void InsetText::rejectChanges()
 {
-	text_.rejectChanges(bparams);
+	text_.rejectChanges();
 }
 
 
-int InsetText::latex(odocstream & os, OutputParams const & runparams) const
+void InsetText::validate(LaTeXFeatures & features) const
 {
-	TexRow texrow;
-	latexParagraphs(buffer(), text_, os, texrow, runparams);
-	return texrow.rows();
+	features.useInsetLayout(getLayout());
+	for_each(paragraphs().begin(), paragraphs().end(),
+		 bind(&Paragraph::validate, _1, ref(features)));
+}
+
+
+void InsetText::latex(otexstream & os, OutputParams const & runparams) const
+{
+	// This implements the standard way of handling the LaTeX
+	// output of a text inset, either a command or an
+	// environment. Standard collapsable insets should not
+	// redefine this, non-standard ones may call this.
+	InsetLayout const & il = getLayout();
+	if (!il.latexname().empty()) {
+		if (il.latextype() == InsetLayout::COMMAND) {
+			// FIXME UNICODE
+			if (runparams.moving_arg)
+				os << "\\protect";
+			os << '\\' << from_utf8(il.latexname());
+			if (!il.latexparam().empty())
+				os << from_utf8(il.latexparam());
+			os << '{';
+		} else if (il.latextype() == InsetLayout::ENVIRONMENT) {
+			if (il.isDisplay())
+			    os << breakln;
+			else
+			    os << safebreakln;
+			if (runparams.lastid != -1)
+				os.texrow().start(runparams.lastid,
+						  runparams.lastpos);
+			os << "\\begin{" << from_utf8(il.latexname()) << "}\n";
+			if (!il.latexparam().empty())
+				os << from_utf8(il.latexparam());
+		}
+	}
+	OutputParams rp = runparams;
+	if (il.isPassThru())
+		rp.pass_thru = true;
+	if (il.isNeedProtect())
+		rp.moving_arg = true;
+	rp.par_begin = 0;
+	rp.par_end = paragraphs().size();
+
+	// Output the contents of the inset
+	latexParagraphs(buffer(), text_, os, rp);
+	runparams.encoding = rp.encoding;
+
+	if (!il.latexname().empty()) {
+		if (il.latextype() == InsetLayout::COMMAND) {
+			os << "}";
+		} else if (il.latextype() == InsetLayout::ENVIRONMENT) {
+			// A comment environment doesn't need a % before \n\end
+			if (il.isDisplay() || runparams.inComment)
+			    os << breakln;
+			else
+			    os << safebreakln;
+			os << "\\end{" << from_utf8(il.latexname()) << "}\n";
+			if (!il.isDisplay())
+				os.protectSpace(true);
+		}
+	}
 }
 
 
@@ -345,15 +476,85 @@ int InsetText::plaintext(odocstream & os, OutputParams const & runparams) const
 
 int InsetText::docbook(odocstream & os, OutputParams const & runparams) const
 {
-	docbookParagraphs(paragraphs(), buffer(), os, runparams);
+	ParagraphList::const_iterator const beg = paragraphs().begin();
+
+	if (!undefined())
+		sgml::openTag(os, getLayout().latexname(),
+			      beg->getID(buffer(), runparams) + getLayout().latexparam());
+
+	docbookParagraphs(text_, buffer(), os, runparams);
+
+	if (!undefined())
+		sgml::closeTag(os, getLayout().latexname());
+
 	return 0;
 }
 
 
-void InsetText::validate(LaTeXFeatures & features) const
+docstring InsetText::xhtml(XHTMLStream & xs, OutputParams const & runparams) const
 {
-	for_each(paragraphs().begin(), paragraphs().end(),
-		 bind(&Paragraph::validate, _1, ref(features)));
+	return insetAsXHTML(xs, runparams, WriteEverything);
+}
+
+
+// FIXME XHTML
+// There are cases where we may need to close open fonts and such
+// and then re-open them when we are done. This would be the case, e.g.,
+// if we were otherwise about to write:
+//		<em>word <div class='foot'>footnote text.</div> emph</em>
+// The problem isn't so much that the footnote text will get emphasized:
+// we can handle that with CSS. The problem is that this is invalid XHTML.
+// One solution would be to make the footnote <span>, but the problem is
+// completely general, and so we'd have to make absolutely everything into
+// span. What I think will work is to check if we're about to write "div" and,
+// if so, try to close fonts, etc. 
+// There are probably limits to how well we can do here, though, and we will
+// have to rely upon users not putting footnotes inside noun-type insets.
+docstring InsetText::insetAsXHTML(XHTMLStream & xs, OutputParams const & rp,
+                                  XHTMLOptions opts) const
+{
+	// we will always want to output all our paragraphs when we are
+	// called this way.
+	OutputParams runparams = rp;
+	runparams.par_begin = 0;
+	runparams.par_end = text().paragraphs().size();
+	
+	if (undefined()) {
+		xhtmlParagraphs(text_, buffer(), xs, runparams);
+		return docstring();
+	}
+
+	InsetLayout const & il = getLayout();
+	if (opts & WriteOuterTag)
+		xs << html::StartTag(il.htmltag(), il.htmlattr());
+	if ((opts & WriteLabel) && !il.counter().empty()) {
+		BufferParams const & bp = buffer().masterBuffer()->params();
+		Counters & cntrs = bp.documentClass().counters();
+		cntrs.step(il.counter(), OutputUpdate);
+		// FIXME: translate to paragraph language
+		if (!il.htmllabel().empty()) {
+			docstring const lbl = 
+				cntrs.counterLabel(from_utf8(il.htmllabel()), bp.language->code());
+			// FIXME is this check necessary?
+			if (!lbl.empty()) {
+				xs << html::StartTag(il.htmllabeltag(), il.htmllabelattr());
+				xs << lbl;
+				xs << html::EndTag(il.htmllabeltag());
+			}
+		}
+	}
+
+	if (opts & WriteInnerTag)
+		xs << html::StartTag(il.htmlinnertag(), il.htmlinnerattr());
+	OutputParams ours = runparams;
+	if (!il.isMultiPar() || opts == JustText)
+		ours.html_make_pars = false;
+	xhtmlParagraphs(text_, buffer(), xs, ours);
+	if (opts & WriteInnerTag)
+		xs << html::EndTag(il.htmlinnertag());
+	if (opts & WriteOuterTag)
+		xs << html::EndTag(il.htmltag());
+	return docstring();
 }
 
 
@@ -365,12 +566,6 @@ void InsetText::cursorPos(BufferView const & bv,
 }
 
 
-bool InsetText::showInsetDialog(BufferView *) const
-{
-	return false;
-}
-
-
 void InsetText::setText(docstring const & data, Font const & font, bool trackChanges)
 {
 	clear();
@@ -473,40 +668,71 @@ ParagraphList & InsetText::paragraphs()
 }
 
 
-void InsetText::updateLabels(ParIterator const & it)
+void InsetText::updateBuffer(ParIterator const & it, UpdateType utype)
 {
 	ParIterator it2 = it;
 	it2.forwardPos();
-	LASSERT(&it2.inset() == this && it2.pit() == 0, /**/);
-	if (producesOutput())
-		lyx::updateLabels(buffer(), it2);
-	else {
+	LASSERT(&it2.inset() == this && it2.pit() == 0, return);
+	if (producesOutput()) {
+		InsetLayout const & il = getLayout();
+		bool const save_layouts = utype == OutputUpdate && il.htmlisblock();
+		Counters & cnt = buffer().masterBuffer()->params().documentClass().counters();
+		if (save_layouts) {
+			// LYXERR0("Entering " << name());
+			cnt.clearLastLayout();
+			// FIXME cnt.saveLastCounter()?
+		}
+		buffer().updateBuffer(it2, utype);
+		if (save_layouts) {
+			// LYXERR0("Exiting " << name());
+			cnt.restoreLastLayout();
+			// FIXME cnt.restoreLastCounter()?
+		}
+	} else {
 		DocumentClass const & tclass = buffer().masterBuffer()->params().documentClass();
+		// Note that we do not need to call:
+		//	tclass.counters().clearLastLayout()
+		// since we are saving and restoring the existing counters, etc.
 		Counters const savecnt = tclass.counters();
-		lyx::updateLabels(buffer(), it2);
+		tclass.counters().reset();
+		buffer().updateBuffer(it2, utype);
 		tclass.counters() = savecnt;
 	}
 }
 
 
-void InsetText::addToToc(DocIterator const & cdit)
+void InsetText::toString(odocstream & os) const
+{
+	os << text().asString(0, 1, AS_STR_LABEL | AS_STR_INSETS);
+}
+
+
+void InsetText::forToc(docstring & os, size_t maxlen) const
+{
+	if (!getLayout().isInToc())
+		return;
+	text().forToc(os, maxlen, false);
+}
+
+
+void InsetText::addToToc(DocIterator const & cdit) const
 {
 	DocIterator dit = cdit;
-	dit.push_back(CursorSlice(*this));
+	dit.push_back(CursorSlice(const_cast<InsetText &>(*this)));
 	Toc & toc = buffer().tocBackend().toc("tableofcontents");
 
 	BufferParams const & bufparams = buffer_->params();
-	const int min_toclevel = bufparams.documentClass().min_toclevel();
+	int const min_toclevel = bufparams.documentClass().min_toclevel();
 
 	// For each paragraph, traverse its insets and let them add
 	// their toc items
-	ParagraphList & pars = paragraphs();
+	ParagraphList const & pars = paragraphs();
 	pit_type pend = paragraphs().size();
 	for (pit_type pit = 0; pit != pend; ++pit) {
 		Paragraph const & par = pars[pit];
 		dit.pit() = pit;
-		// the string that goes to the toc (could be the optarg)
-		docstring tocstring;
+		// if we find an optarg, we'll save it for use later.
+		InsetText const * arginset = 0;
 		InsetList::const_iterator it  = par.insetList().begin();
 		InsetList::const_iterator end = par.insetList().end();
 		for (; it != end; ++it) {
@@ -514,29 +740,22 @@ void InsetText::addToToc(DocIterator const & cdit)
 			dit.pos() = it->pos;
 			//lyxerr << (void*)&inset << " code: " << inset.lyxCode() << std::endl;
 			inset.addToToc(dit);
-			switch (inset.lyxCode()) {
-			case OPTARG_CODE: {
-				if (!tocstring.empty())
-					break;
-				dit.pos() = 0;
-				Paragraph const & insetpar =
-					*static_cast<InsetOptArg&>(inset).paragraphs().begin();
-				if (!par.labelString().empty())
-					tocstring = par.labelString() + ' ';
-				tocstring += insetpar.asString(AS_STR_INSETS);
-				break;
-			}
-			default:
-				break;
-			}
+			if (inset.lyxCode() == ARG_CODE)
+				arginset = inset.asInsetText();
 		}
-		/// now the toc entry for the paragraph
+		// now the toc entry for the paragraph
 		int const toclevel = par.layout().toclevel;
 		if (toclevel != Layout::NOT_IN_TOC && toclevel >= min_toclevel) {
-			dit.pos() = 0;
 			// insert this into the table of contents
-			if (tocstring.empty())
-				tocstring = par.asString(AS_STR_LABEL | AS_STR_INSETS);
+			docstring tocstring;
+			if (arginset) {
+				tocstring = par.labelString();
+				if (!tocstring.empty())
+					tocstring += ' ';
+				arginset->text().forToc(tocstring, TOC_ENTRY_LENGTH);
+			} else
+				par.forToc(tocstring, TOC_ENTRY_LENGTH);
+			dit.pos() = 0;
 			toc.push_back(TocItem(dit, toclevel - min_toclevel,
 				tocstring, tocstring));
 		}
@@ -549,7 +768,7 @@ void InsetText::addToToc(DocIterator const & cdit)
 
 bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 {
-	if (cur.buffer().isClean())
+	if (buffer().isClean())
 		return Inset::notifyCursorLeaves(old, cur);
 	
 	// find text inset in old cursor
@@ -560,7 +779,7 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 	LASSERT(&insetCur.inset() == this, /**/);
 	
 	// update the old paragraph's words
-	insetCur.paragraph().updateWords(insetCur.top());
+	insetCur.paragraph().updateWords();
 	
 	return Inset::notifyCursorLeaves(old, cur);
 }
@@ -568,9 +787,7 @@ bool InsetText::notifyCursorLeaves(Cursor const & old, Cursor & cur)
 
 bool InsetText::completionSupported(Cursor const & cur) const
 {
-	Cursor const & bvCur = cur.bv().cursor();
-	if (&bvCur.inset() != this)
-		return false;
+	//LASSERT(&cur.bv().cursor().inset() != this, return false);
 	return text_.completionSupported(cur);
 }
 
@@ -632,22 +849,95 @@ void InsetText::completionPosAndDim(Cursor const & cur, int & x, int & y,
 
 
 docstring InsetText::contextMenu(BufferView const &, int, int) const
+{
+	docstring context_menu = contextMenuName();
+	if (context_menu != InsetText::contextMenuName())
+		context_menu += ";" + InsetText::contextMenuName(); 
+	return context_menu;
+}
+
+
+docstring InsetText::contextMenuName() const
 {
 	return from_ascii("context-edit");
 }
 
 
-
-docstring InsetText::toolTipText() const
+docstring InsetText::toolTipText(docstring prefix,
+		size_t numlines, size_t len) const
 {
+	size_t const max_length = numlines * len;
 	OutputParams rp(&buffer().params().encoding());
-	odocstringstream ods;
-	// do not remove InsetText::, otherwise there
-	// will be no tooltip text for InsetNotes
-	InsetText::plaintext(ods, rp);
-	docstring const content_tip = ods.str();
-	return support::wrapParas(content_tip, 4);
+	odocstringstream oss;
+	oss << prefix;
+
+	ParagraphList::const_iterator beg = paragraphs().begin();
+	ParagraphList::const_iterator end = paragraphs().end();
+	ParagraphList::const_iterator it = beg;
+	bool ref_printed = false;
+	docstring str;
+
+	for (; it != end; ++it) {
+		if (it != beg)
+			oss << '\n';
+		writePlaintextParagraph(buffer(), *it, oss, rp, ref_printed);
+		str = oss.str();
+		if (str.length() > max_length)
+			break;
+	}
+	return support::wrapParas(str, 4, len, numlines);
 }
 
 
+InsetCaption const * InsetText::getCaptionInset() const
+{
+	ParagraphList::const_iterator pit = paragraphs().begin();
+	for (; pit != paragraphs().end(); ++pit) {
+		InsetList::const_iterator it = pit->insetList().begin();
+		for (; it != pit->insetList().end(); ++it) {
+			Inset & inset = *it->inset;
+			if (inset.lyxCode() == CAPTION_CODE) {
+				InsetCaption const * ins =
+					static_cast<InsetCaption const *>(it->inset);
+				return ins;
+			}
+		}
+	}
+	return 0;
+}
+
+
+docstring InsetText::getCaptionText(OutputParams const & runparams) const
+{
+	InsetCaption const * ins = getCaptionInset();
+	if (ins == 0)
+		return docstring();
+
+	odocstringstream ods;
+	ins->getCaptionAsPlaintext(ods, runparams);
+	return ods.str();
+}
+
+
+docstring InsetText::getCaptionHTML(OutputParams const & runparams) const
+{
+	InsetCaption const * ins = getCaptionInset();
+	if (ins == 0)
+		return docstring();
+
+	odocstringstream ods;
+	XHTMLStream xs(ods);
+	docstring def = ins->getCaptionAsHTML(xs, runparams);
+	if (!def.empty())
+		// should already have been escaped
+		xs << XHTMLStream::ESCAPE_NONE << def << '\n';
+	return ods.str();
+}
+
+
+InsetText::XHTMLOptions operator|(InsetText::XHTMLOptions a1, InsetText::XHTMLOptions a2)
+{
+	return static_cast<InsetText::XHTMLOptions>((int)a1 | (int)a2);
+}
+
 } // namespace lyx
