@@ -12,19 +12,19 @@
 #ifndef LYX_DOCSTREAM_H
 #define LYX_DOCSTREAM_H
 
+#include "TexRow.h"
 #include "support/docstring.h"
 
-#include <fstream>
-#include <sstream>
-
 #if defined(_MSC_VER) && (_MSC_VER >= 1600) 
-// Ugly workaround for MSVC10 STL bug
-// http://connect.microsoft.com/VisualStudio/feedback/details/572376/msvc10-c-std-numpunct-has-a-hardcoded-dllimport-in-definition
+// Ugly workaround for MSVC10 STL bug:
 // std::numpunct has a hardcoded dllimport in definition, but we wanna it with 32 bit 
 // so we can't import it and must define it but then the compiler complains.
 #include "support/numpunct_lyx_char_type.h"
 #endif
 
+#include <fstream>
+#include <sstream>
+
 namespace lyx {
 
 class iconv_codecvt_facet_exception : public std::exception {
@@ -42,9 +42,9 @@ typedef std::basic_istream<char_type> idocstream;
     \endcode, not \code
     os << c;
     \endcode . The latter will not output the character, but the code point
-    as number. This is because we can't overload operator<< (our character
-    type is not a real type but a typedef). Narrow characters of type char
-    can be output as usual.
+    as number if USE_WCHAR_T is not defined. This is because we can't overload
+    operator<< (our character type is not always a real type but sometimes a
+    typedef). Narrow characters of type char can be output as usual.
  */
 typedef std::basic_ostream<char_type> odocstream;
 
@@ -53,7 +53,7 @@ typedef std::basic_ostream<char_type> odocstream;
 class ifdocstream : public std::basic_ifstream<char_type> {
 	typedef std::basic_ifstream<char_type> base;
 public:
-	ifdocstream(std::string const & encoding = "UTF-8");
+	ifdocstream();
 	explicit ifdocstream(const char* s,
 		std::ios_base::openmode mode = std::ios_base::in,
 		std::string const & encoding = "UTF-8");
@@ -62,6 +62,7 @@ public:
 
 /// File stream for writing files in 8bit encoding \p encoding with automatic
 /// conversion from UCS4.
+
 class ofdocstream : public std::basic_ofstream<char_type> {
 	typedef std::basic_ofstream<char_type> base;
 public:
@@ -74,12 +75,85 @@ public:
 	void reset(std::string const & encoding);
 };
 
+
+
 /// UCS4 input stringstream
 typedef std::basic_istringstream<char_type> idocstringstream;
 
 /// UCS4 output stringstream
 typedef std::basic_ostringstream<char_type> odocstringstream;
 
+/// UCS4 output manipulator
+typedef odocstream & (*odocstream_manip)(odocstream &);
+
+/** Wrapper class for odocstream.
+    This class is used to automatically count the lines of the exported latex
+    code and also to ensure that no blank lines may be inadvertently output.
+    To this end, use the special variables "breakln" and "safebreakln" as if
+    they were iomanip's to ensure that the next output will start at the
+    beginning of a line. Using "breakln", a '\n' char will be output if needed,
+    while using "safebreakln", "%\n" will be output if needed.
+  */
+
+class otexstream {
+public:
+	///
+	otexstream(odocstream & os, TexRow & texrow)
+		: os_(os), texrow_(texrow),
+		  canbreakline_(false), protectspace_(false) {}
+	///
+	odocstream & os() { return os_; }
+	///
+	TexRow & texrow() { return texrow_; }
+	///
+	void put(char_type const & c);
+	///
+	void canBreakLine(bool breakline) { canbreakline_ = breakline; }
+	///
+	bool canBreakLine() const { return canbreakline_; }
+	///
+	void protectSpace(bool protectspace) { protectspace_ = protectspace; }
+	///
+	bool protectSpace() const { return protectspace_; }
+private:
+	///
+	odocstream & os_;
+	///
+	TexRow & texrow_;
+	///
+	bool canbreakline_;
+	///
+	bool protectspace_;
+};
+
+/// Helper structs for breaking a line
+struct BreakLine {
+	char n;
+};
+
+struct SafeBreakLine {
+	char n;
+};
+
+extern BreakLine breakln;
+extern SafeBreakLine safebreakln;
+
+///
+otexstream & operator<<(otexstream &, BreakLine);
+///
+otexstream & operator<<(otexstream &, SafeBreakLine);
+///
+otexstream & operator<<(otexstream &, odocstream_manip);
+///
+otexstream & operator<<(otexstream &, docstring const &);
+///
+otexstream & operator<<(otexstream &, char const *);
+///
+otexstream & operator<<(otexstream &, char);
+///
+template <typename Type>
+otexstream & operator<<(otexstream & ots, Type value);
+
 /// Helper struct for changing stream encoding
 struct SetEnc {
 	SetEnc(std::string const & e) : encoding(e) {}
