@@ -3,8 +3,8 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Andr Pnitz
- * \author Uwe Sthr
+ * \author André Pönitz
+ * \author Uwe Stöhr
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -25,7 +25,7 @@
 #include "support/filetools.h"
 #include "support/lstrings.h"
 
-#include <boost/regex.hpp>
+#include "support/regex.h"
 
 #include <algorithm>
 #include <iostream>
@@ -36,8 +36,7 @@
 
 using namespace std;
 using namespace lyx::support;
-using boost::regex;
-using boost::smatch;
+
 
 namespace lyx {
 
@@ -45,56 +44,84 @@ namespace lyx {
 extern map<char, int> special_columns;
 
 map<string, vector<string> > used_packages;
+const char * const modules_placeholder = "\001modules\001";
 
 // needed to handle encodings with babel
 bool one_language = true;
 
 namespace {
 
+//add this to known_languages when updating to lyxformat 266:
+// "armenian"
 //add these to known_languages when updating to lyxformat 268:
 //"chinese-simplified", "chinese-traditional", "japanese", "korean"
-const char * const known_languages[] = { "afrikaans", "american", "arabic",
-"austrian", "bahasa", "basque", "belarusian", "brazil", "brazilian", "breton",
-"british", "bulgarian", "canadian", "canadien", "catalan", "croatian", "czech",
-"danish", "dutch", "english", "esperanto", "estonian", "finnish", "francais",
-"french", "frenchb", "frenchle", "frenchpro", "galician", "german", "germanb",
-"greek", "hebrew", "icelandic", "irish", "italian", "kazakh", "lsorbian", "magyar",
-"naustrian", "ngerman", "ngermanb", "norsk", "nynorsk", "polish", "portuges",
-"portuguese", "romanian", "russian", "russianb", "scottish", "serbian", "slovak",
-"slovene", "spanish", "swedish", "thai", "turkish", "ukraineb", "ukrainian",
-"usorbian", "welsh", 0};
+// Both changes require first that support for non-babel languages (CJK,
+// armtex) is added.
+// add turkmen for lyxformat 383
+/**
+ * known babel language names (including synonyms)
+ * not in standard babel: arabic, arabtex, belarusian, serbian-latin, thai
+ * not yet supported by LyX: kurmanji
+ * please keep this in sync with known_coded_languages line by line!
+ */
+const char * const known_languages[] = {"acadian", "afrikaans", "albanian",
+"american", "arabic", "arabtex", "austrian", "bahasa", "bahasai", "bahasam",
+"basque", "belarusian", "brazil", "brazilian", "breton", "british", "bulgarian",
+"canadian", "canadien", "catalan", "croatian", "czech", "danish", "dutch",
+"english", "esperanto", "estonian", "farsi", "finnish", "francais", "french",
+"frenchb", "frenchle", "frenchpro", "galician", "german", "germanb", "greek",
+"hebrew", "hungarian", "icelandic", "indon", "indonesian", "interlingua",
+"irish", "italian", "kazakh", "latin", "latvian", "lithuanian", "lowersorbian",
+"lsorbian", "magyar", "malay", "meyalu", "mongolian", "naustrian", "newzealand",
+"ngerman", "ngermanb", "norsk", "nynorsk", "polutonikogreek", "polish",
+"portuges", "portuguese", "romanian", "russian", "russianb", "samin",
+"scottish", "serbian", "serbian-latin", "slovak", "slovene", "spanish",
+"swedish", "thai", "turkish", "ukraineb", "ukrainian", "uppersorbian",
+"UKenglish", "USenglish", "usorbian", "vietnam", "welsh", 0};
 
-//add this when updating to lyxformat 305:
-//bahasai, indonesian, and indon = equal to bahasa
-//malay and meyalu = equal to bahasam
-const char * const known_brazilian_languages[] = {"brazil", "brazilian", 0};
-const char * const known_french_languages[] = {"french", "frenchb", "francais",
-						"frenchle", "frenchpro", 0};
-const char * const known_german_languages[] = {"german", "germanb", 0};
-const char * const known_ngerman_languages[] = {"ngerman", "ngermanb", 0};
-const char * const known_portuguese_languages[] = {"portuges", "portuguese", 0};
-const char * const known_russian_languages[] = {"russian", "russianb", 0};
-const char * const known_ukrainian_languages[] = {"ukrainian", "ukraineb", 0};
+/**
+ * the same as known_languages with .lyx names
+ * please keep this in sync with known_languages line by line!
+ */
+const char * const known_coded_languages[] = {"french", "afrikaans", "albanian",
+"american", "arabic_arabi", "arabic_arabtex", "austrian", "bahasa", "bahasa", "bahasam",
+"basque", "belarusian", "brazilian", "brazilian", "breton", "british", "bulgarian",
+"canadian", "canadien", "catalan", "croatian", "czech", "danish", "dutch",
+"english", "esperanto", "estonian", "farsi", "finnish", "french", "french",
+"french", "french", "french", "galician", "german", "german", "greek",
+"hebrew", "magyar", "icelandic", "bahasa", "bahasa", "interlingua",
+"irish", "italian", "kazakh", "latin", "latvian", "lithuanian", "lowersorbian",
+"lowersorbian", "magyar", "bahasam", "bahasam", "mongolian", "naustrian", "english",
+"ngerman", "ngerman", "norsk", "nynorsk", "polutonikogreek", "polish",
+"portuguese", "portuguese", "romanian", "russian", "russian", "samin",
+"scottish", "serbian", "serbian-latin", "slovak", "slovene", "spanish",
+"swedish", "thai", "turkish", "ukrainian", "ukrainian", "uppersorbian",
+"uppersorbian", "english", "english", "vietnamese", "welsh", 0};
 
-//add these to known_english_quotes_languages when updating to lyxformat 268:
-//"chinese-simplified", "korean"
-const char * const known_english_quotes_languages[] = {"american", "canadian",
-"english", "esperanto", "hebrew", "irish", "scottish", "thai", 0};
+/// languages with english quotes (.lyx names)
+const char * const known_english_quotes_languages[] = {"american", "bahasa",
+"bahasam", "brazilian", "canadian", "chinese-simplified", "english",
+"esperanto", "hebrew", "irish", "korean", "portuguese", "scottish", "thai", 0};
 
-//add this to known_french_quotes_languages when updating to lyxformat 327:
-//"spanish-mexico"
-const char * const known_french_quotes_languages[] = {"albanian", "arabic",
-"basque", "canadien", "catalan", "galician", "greek", "italian", "norsk",
-"nynorsk", "spanish", "turkish", 0};
+//add this to known_french_quotes_languages when updating to
+//lyxformat 383: "turkmen"
+/// languages with french quotes (.lyx names)
+const char * const known_french_quotes_languages[] = {"albanian",
+"arabic_arabi", "arabic_arabtex", "basque", "canadien", "catalan", "french",
+"galician", "greek", "italian", "norsk", "nynorsk", "polutonikogreek",
+"russian", "spanish", "spanish-mexico", "turkish", "ukrainian", "vietnamese", 0};
 
+/// languages with german quotes (.lyx names)
 const char * const known_german_quotes_languages[] = {"austrian", "bulgarian",
-"czech", "icelandic", "lithuanian", "lsorbian", "naustrian", "serbian",
-"serbian-latin", "slovak", "slovene", "usorbian",  0};
+"czech", "german", "icelandic", "lithuanian", "lowersorbian", "naustrian",
+"ngerman", "serbian", "serbian-latin", "slovak", "slovene", "uppersorbian", 0};
 
+/// languages with polish quotes (.lyx names)
 const char * const known_polish_quotes_languages[] = {"afrikaans", "croatian",
 "dutch", "estonian", "magyar", "polish", "romanian", 0};
 
-const char * const known_swedish_quotes_languages[] = {"bahasa", "finnish", 
+/// languages with swedish quotes (.lyx names)
+const char * const known_swedish_quotes_languages[] = {"finnish",
 "swedish", 0};
 
 char const * const known_fontsizes[] = { "10pt", "11pt", "12pt", 0 };
@@ -123,10 +150,20 @@ const char * const known_coded_paper_margins[] = { "leftmargin", "topmargin",
 "rightmargin", "bottommargin", "headheight", "headsep", "footskip",
 "columnsep", 0};
 
+/// commands that can start an \if...\else...\endif sequence
+const char * const known_if_commands[] = {"if", "ifarydshln", "ifbraket",
+"ifcancel", "ifcolortbl", "ifeurosym", "ifmarginnote", "ifmmode", "ifpdf",
+"ifsidecap", "ifupgreek", 0};
+
+/// conditional commands with three arguments like \@ifundefined{}{}{}
+const char * const known_if_3arg_commands[] = {"@ifundefined", "IfFileExists",
+0};
+
 // default settings
 ostringstream h_preamble;
 string h_textclass               = "article";
-string h_options                 = string();
+string h_use_default_options     = "false";
+string h_options;
 string h_language                = "english";
 string h_inputencoding           = "auto";
 string h_font_roman              = "default";
@@ -138,8 +175,25 @@ string h_font_osf                = "false";
 string h_font_sf_scale           = "100";
 string h_font_tt_scale           = "100";
 string h_graphics                = "default";
+string h_float_placement;
 string h_paperfontsize           = "default";
 string h_spacing                 = "single";
+string h_use_hyperref            = "0";
+string h_pdf_title;
+string h_pdf_author;
+string h_pdf_subject;
+string h_pdf_keywords;
+string h_pdf_bookmarks           = "1";
+string h_pdf_bookmarksnumbered   = "0";
+string h_pdf_bookmarksopen       = "0";
+string h_pdf_bookmarksopenlevel  = "1";
+string h_pdf_breaklinks          = "0";
+string h_pdf_pdfborder           = "0";
+string h_pdf_colorlinks          = "0";
+string h_pdf_backref             = "section";
+string h_pdf_pdfusetitle         = "1";
+string h_pdf_pagemode;
+string h_pdf_quoted_options;
 string h_papersize               = "default";
 string h_use_geometry            = "false";
 string h_use_amsmath             = "1";
@@ -147,24 +201,29 @@ string h_use_esint               = "1";
 string h_cite_engine             = "basic";
 string h_use_bibtopic            = "false";
 string h_paperorientation        = "portrait";
+string h_notefontcolor;
 string h_secnumdepth             = "3";
 string h_tocdepth                = "3";
 string h_paragraph_separation    = "indent";
 string h_defskip                 = "medskip";
+string h_paragraph_indentation   = "default";
 string h_quotes_language         = "english";
 string h_papercolumns            = "1";
-string h_papersides              = string();
+string h_papersides;
 string h_paperpagestyle          = "default";
+string h_listings_params;
 string h_tracking_changes        = "false";
 string h_output_changes          = "false";
-string h_margins                 = "";
+string h_margins;
 
 
-void handle_opt(vector<string> & opts, char const * const * what, string & target)
+// returns true if at least one of the options in what has been found
+bool handle_opt(vector<string> & opts, char const * const * what, string & target)
 {
 	if (opts.empty())
-		return;
+		return false;
 
+	bool found = false;
 	// the last language option is the document language (for babel and LyX)
 	// the last size option is the document font size
 	vector<string>::iterator it;
@@ -173,11 +232,13 @@ void handle_opt(vector<string> & opts, char const * const * what, string & targe
 		it = find(opts.begin(), opts.end(), *what);
 		if (it != opts.end()) {
 			if (it >= position) {
+				found = true;
 				target = *what;
 				position = it;
 			}
 		}
 	}
+	return found;
 }
 
 
@@ -231,6 +292,28 @@ vector<string> split_options(string const & input)
 }
 
 
+/*!
+ * Retrieve a keyval option "name={value with=sign}" named \p name from
+ * \p options and return the value.
+ * The found option is also removed from \p options.
+ */
+string process_keyval_opt(vector<string> & options, string name)
+{
+	for (size_t i = 0; i < options.size(); ++i) {
+		vector<string> option;
+		split(options[i], option, '=');
+		if (option.size() < 2)
+			continue;
+		if (option[0] == name) {
+			options.erase(options.begin() + i);
+			option.erase(option.begin());
+			return join(option, "=");
+		}
+	}
+	return "";
+}
+
+
 /*!
  * Add package \p name with options \p options to used_packages.
  * Remove options from \p options that we don't want to output.
@@ -269,6 +352,100 @@ string const scale_as_percentage(string const & scale)
 }
 
 
+string remove_braces(string const & value)
+{
+	if (value.empty())
+		return value;
+	if (value[0] == '{' && value[value.length()-1] == '}')
+		return value.substr(1, value.length()-2);
+	return value;
+}
+
+
+void handle_hyperref(vector<string> & options)
+{
+	// FIXME swallow inputencoding changes that might surround the
+	//       hyperref setup if it was written by LyX
+	h_use_hyperref = "1";
+	// swallow "unicode=true", since LyX does always write that
+	vector<string>::iterator it =
+		find(options.begin(), options.end(), "unicode=true");
+	if (it != options.end())
+		options.erase(it);
+	it = find(options.begin(), options.end(), "pdfusetitle");
+	if (it != options.end()) {
+		h_pdf_pdfusetitle = "1";
+		options.erase(it);
+	}
+	string bookmarks = process_keyval_opt(options, "bookmarks");
+	if (bookmarks == "true")
+		h_pdf_bookmarks = "1";
+	else if (bookmarks == "false")
+		h_pdf_bookmarks = "0";
+	if (h_pdf_bookmarks == "1") {
+		string bookmarksnumbered =
+			process_keyval_opt(options, "bookmarksnumbered");
+		if (bookmarksnumbered == "true")
+			h_pdf_bookmarksnumbered = "1";
+		else if (bookmarksnumbered == "false")
+			h_pdf_bookmarksnumbered = "0";
+		string bookmarksopen =
+			process_keyval_opt(options, "bookmarksopen");
+		if (bookmarksopen == "true")
+			h_pdf_bookmarksopen = "1";
+		else if (bookmarksopen == "false")
+			h_pdf_bookmarksopen = "0";
+		if (h_pdf_bookmarksopen == "1") {
+			string bookmarksopenlevel =
+				process_keyval_opt(options, "bookmarksopenlevel");
+			if (!bookmarksopenlevel.empty())
+				h_pdf_bookmarksopenlevel = bookmarksopenlevel;
+		}
+	}
+	string breaklinks = process_keyval_opt(options, "breaklinks");
+	if (breaklinks == "true")
+		h_pdf_breaklinks = "1";
+	else if (breaklinks == "false")
+		h_pdf_breaklinks = "0";
+	string pdfborder = process_keyval_opt(options, "pdfborder");
+	if (pdfborder == "{0 0 0}")
+		h_pdf_pdfborder = "1";
+	else if (pdfborder == "{0 0 1}")
+		h_pdf_pdfborder = "0";
+	string backref = process_keyval_opt(options, "backref");
+	if (!backref.empty())
+		h_pdf_backref = backref;
+	string colorlinks = process_keyval_opt(options, "colorlinks");
+	if (colorlinks == "true")
+		h_pdf_colorlinks = "1";
+	else if (colorlinks == "false")
+		h_pdf_colorlinks = "0";
+	string pdfpagemode = process_keyval_opt(options, "pdfpagemode");
+	if (!pdfpagemode.empty())
+		h_pdf_pagemode = pdfpagemode;
+	string pdftitle = process_keyval_opt(options, "pdftitle");
+	if (!pdftitle.empty()) {
+		h_pdf_title = remove_braces(pdftitle);
+	}
+	string pdfauthor = process_keyval_opt(options, "pdfauthor");
+	if (!pdfauthor.empty()) {
+		h_pdf_author = remove_braces(pdfauthor);
+	}
+	string pdfsubject = process_keyval_opt(options, "pdfsubject");
+	if (!pdfsubject.empty())
+		h_pdf_subject = remove_braces(pdfsubject);
+	string pdfkeywords = process_keyval_opt(options, "pdfkeywords");
+	if (!pdfkeywords.empty())
+		h_pdf_keywords = remove_braces(pdfkeywords);
+	if (!options.empty()) {
+		if (!h_pdf_quoted_options.empty())
+			h_pdf_quoted_options += ',';
+		h_pdf_quoted_options += join(options, ",");
+		options.clear();
+	}
+}
+
+
 void handle_package(Parser &p, string const & name, string const & opts,
 		    bool in_lyx_preamble)
 {
@@ -375,6 +552,9 @@ void handle_package(Parser &p, string const & name, string const & opts,
 	else if (name == "url")
 		; // ignore this
 
+	else if (LYX_FORMAT >= 408 && name == "subscript")
+		; // ignore this
+
 	else if (name == "color") {
 		// with the following command this package is only loaded when needed for
 		// undefined colors, since we only support the predefined colors
@@ -412,6 +592,9 @@ void handle_package(Parser &p, string const & name, string const & opts,
 	else if (name == "jurabib")
 		h_cite_engine = "jurabib";
 
+	else if (name == "hyperref")
+		handle_hyperref(options);
+
 	else if (!in_lyx_preamble) {
 		if (options.empty())
 			h_preamble << "\\usepackage{" << name << "}";
@@ -432,24 +615,27 @@ void handle_package(Parser &p, string const & name, string const & opts,
 }
 
 
+void handle_if(Parser & p, bool in_lyx_preamble)
+{
+	while (p.good()) {
+		Token t = p.get_token();
+		if (t.cat() == catEscape &&
+		    is_known(t.cs(), known_if_commands))
+			handle_if(p, in_lyx_preamble);
+		else {
+			if (!in_lyx_preamble)
+				h_preamble << t.asInput();
+			if (t.cat() == catEscape && t.cs() == "fi")
+				return;
+		}
+	}
+}
+
 
 void end_preamble(ostream & os, TextClass const & /*textclass*/)
 {
-	// merge synonym languages
-	if (is_known(h_language, known_brazilian_languages))
-		h_language = "brazilian";
-	else if (is_known(h_language, known_french_languages))
-		h_language = "french";
-	else if (is_known(h_language, known_german_languages))
-		h_language = "german";
-	else if (is_known(h_language, known_ngerman_languages))
-		h_language = "ngerman";
-	else if (is_known(h_language, known_portuguese_languages))
-		h_language = "portuguese";
-	else if (is_known(h_language, known_russian_languages))
-		h_language = "russian";
-	else if (is_known(h_language, known_ukrainian_languages))
-		h_language = "ukrainian";
+	// translate from babel to LyX names
+	h_language = babel2lyx(h_language);
 
 	// set the quote language
 	// LyX only knows the following quotes languages:
@@ -463,15 +649,10 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 	if (h_language == "danish")
 		h_quotes_language = "danish";
 	// french
-	else if (is_known(h_language, known_french_quotes_languages)
-		|| is_known(h_language, known_french_languages)
-		|| is_known(h_language, known_russian_languages)
-		|| is_known(h_language, known_ukrainian_languages))
+	else if (is_known(h_language, known_french_quotes_languages))
 		h_quotes_language = "french";
 	// german
-	else if (is_known(h_language, known_german_quotes_languages)
-		|| is_known(h_language, known_german_languages)
-		|| is_known(h_language, known_ngerman_languages))
+	else if (is_known(h_language, known_german_quotes_languages))
 		h_quotes_language = "german";
 	// polish
 	else if (is_known(h_language, known_polish_quotes_languages))
@@ -480,14 +661,12 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 	else if (is_known(h_language, known_swedish_quotes_languages))
 		h_quotes_language = "swedish";
 	//english
-	else if (is_known(h_language, known_english_quotes_languages)
-		|| is_known(h_language, known_brazilian_languages)
-		|| is_known(h_language, known_portuguese_languages))
+	else if (is_known(h_language, known_english_quotes_languages))
 		h_quotes_language = "english";
 
 	// output the LyX file settings
 	os << "#LyX file created by tex2lyx " << PACKAGE_VERSION << "\n"
-	   << "\\lyxformat 264\n"
+	   << "\\lyxformat " << LYX_FORMAT << '\n'
 	   << "\\begin_document\n"
 	   << "\\begin_header\n"
 	   << "\\textclass " << h_textclass << "\n";
@@ -495,7 +674,9 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 		os << "\\begin_preamble\n" << h_preamble.str() << "\n\\end_preamble\n";
 	if (!h_options.empty())
 		os << "\\options " << h_options << "\n";
-	os << "\\language " << h_language << "\n"
+	os << "\\use_default_options " << h_use_default_options << "\n"
+	   << modules_placeholder
+	   << "\\language " << h_language << "\n"
 	   << "\\inputencoding " << h_inputencoding << "\n"
 	   << "\\font_roman " << h_font_roman << "\n"
 	   << "\\font_sans " << h_font_sans << "\n"
@@ -505,26 +686,59 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 	   << "\\font_osf " << h_font_osf << "\n"
 	   << "\\font_sf_scale " << h_font_sf_scale << "\n"
 	   << "\\font_tt_scale " << h_font_tt_scale << "\n"
-	   << "\\graphics " << h_graphics << "\n"
-	   << "\\paperfontsize " << h_paperfontsize << "\n"
+	   << "\\graphics " << h_graphics << "\n";
+	if (!h_float_placement.empty())
+		os << "\\float_placement " << h_float_placement << "\n";
+	os << "\\paperfontsize " << h_paperfontsize << "\n"
 	   << "\\spacing " << h_spacing << "\n"
-	   << "\\papersize " << h_papersize << "\n"
+	   << "\\use_hyperref " << h_use_hyperref << '\n';
+	if (h_use_hyperref == "1") {
+		if (!h_pdf_title.empty())
+			os << "\\pdf_title \"" << h_pdf_title << "\"\n";
+		if (!h_pdf_author.empty())
+			os << "\\pdf_author \"" << h_pdf_author << "\"\n";
+		if (!h_pdf_subject.empty())
+			os << "\\pdf_subject \"" << h_pdf_subject << "\"\n";
+		if (!h_pdf_keywords.empty())
+			os << "\\pdf_keywords \"" << h_pdf_keywords << "\"\n";
+		os << "\\pdf_bookmarks " << h_pdf_bookmarks << "\n"
+		      "\\pdf_bookmarksnumbered " << h_pdf_bookmarksnumbered << "\n"
+		      "\\pdf_bookmarksopen " << h_pdf_bookmarksopen << "\n"
+		      "\\pdf_bookmarksopenlevel " << h_pdf_bookmarksopenlevel << "\n"
+		      "\\pdf_breaklinks " << h_pdf_breaklinks << "\n"
+		      "\\pdf_pdfborder " << h_pdf_pdfborder << "\n"
+		      "\\pdf_colorlinks " << h_pdf_colorlinks << "\n"
+		      "\\pdf_backref " << h_pdf_backref << "\n"
+		      "\\pdf_pdfusetitle " << h_pdf_pdfusetitle << '\n';
+		if (!h_pdf_pagemode.empty())
+			os << "\\pdf_pagemode " << h_pdf_pagemode << '\n';
+		if (!h_pdf_quoted_options.empty())
+			os << "\\pdf_quoted_options \"" << h_pdf_quoted_options << "\"\n";
+	}
+	os << "\\papersize " << h_papersize << "\n"
 	   << "\\use_geometry " << h_use_geometry << "\n"
 	   << "\\use_amsmath " << h_use_amsmath << "\n"
 	   << "\\use_esint " << h_use_esint << "\n"
 	   << "\\cite_engine " << h_cite_engine << "\n"
 	   << "\\use_bibtopic " << h_use_bibtopic << "\n"
-	   << "\\paperorientation " << h_paperorientation << "\n"
-	   << h_margins
+	   << "\\paperorientation " << h_paperorientation << '\n';
+	if (LYX_FORMAT >= 382 && !h_notefontcolor.empty())
+		os << "\\notefontcolor " << h_notefontcolor << '\n';
+	os << h_margins
 	   << "\\secnumdepth " << h_secnumdepth << "\n"
 	   << "\\tocdepth " << h_tocdepth << "\n"
-	   << "\\paragraph_separation " << h_paragraph_separation << "\n"
-	   << "\\defskip " << h_defskip << "\n"
-	   << "\\quotes_language " << h_quotes_language << "\n"
+	   << "\\paragraph_separation " << h_paragraph_separation << "\n";
+	if (LYX_FORMAT < 365 || h_paragraph_separation == "skip")
+		os << "\\defskip " << h_defskip << "\n";
+	else
+		os << "\\paragraph_indentation " << h_paragraph_indentation << "\n";
+	os << "\\quotes_language " << h_quotes_language << "\n"
 	   << "\\papercolumns " << h_papercolumns << "\n"
 	   << "\\papersides " << h_papersides << "\n"
-	   << "\\paperpagestyle " << h_paperpagestyle << "\n"
-	   << "\\tracking_changes " << h_tracking_changes << "\n"
+	   << "\\paperpagestyle " << h_paperpagestyle << "\n";
+	if (!h_listings_params.empty())
+		os << "\\listings_params " << h_listings_params << "\n";
+	os << "\\tracking_changes " << h_tracking_changes << "\n"
 	   << "\\output_changes " << h_output_changes << "\n"
 	   << "\\end_header\n\n"
 	   << "\\begin_body\n";
@@ -534,6 +748,7 @@ void end_preamble(ostream & os, TextClass const & /*textclass*/)
 
 } // anonymous namespace
 
+
 void parse_preamble(Parser & p, ostream & os, 
 	string const & forceclass, TeX2LyXDocClass & tc)
 {
@@ -584,9 +799,9 @@ void parse_preamble(Parser & p, ostream & os,
 		else if (t.cat() == catComment) {
 			static regex const islyxfile("%% LyX .* created this file");
 			static regex const usercommands("User specified LaTeX commands");
-			
+
 			string const comment = t.asInput();
-			
+
 			// magically switch encoding default if it looks like XeLaTeX
 			static string const magicXeLaTeX =
 				"% This document must be compiled with XeLaTeX ";
@@ -596,7 +811,6 @@ void parse_preamble(Parser & p, ostream & os,
 				cerr << "XeLaTeX comment found, switching to UTF8\n";
 				h_inputencoding = "utf8";
 			}
-
 			smatch sub;
 			if (regex_search(comment, sub, islyxfile)) {
 				is_lyx_file = true;
@@ -612,22 +826,20 @@ void parse_preamble(Parser & p, ostream & os,
 			h_paperpagestyle = p.verbatim_item();
 
 		else if (t.cs() == "makeatletter") {
-			if (!is_lyx_file || !in_lyx_preamble
-			    || p.getCatCode('@') != catLetter)
-				h_preamble << "\\makeatletter";
+			// LyX takes care of this
 			p.setCatCode('@', catLetter);
 		}
 
 		else if (t.cs() == "makeatother") {
-			if (!is_lyx_file || !in_lyx_preamble
-			    || p.getCatCode('@') != catOther)
-				h_preamble << "\\makeatother";
+			// LyX takes care of this
 			p.setCatCode('@', catOther);
 		}
 
-		else if (t.cs() == "newcommand" || t.cs() == "renewcommand"
-			    || t.cs() == "providecommand"
+		else if (t.cs() == "newcommand" || t.cs() == "newcommandx"
+		      || t.cs() == "renewcommand" || t.cs() == "renewcommandx"
+		      || t.cs() == "providecommand" || t.cs() == "providecommandx"
 				|| t.cs() == "DeclareRobustCommand"
+		      || t.cs() == "DeclareRobustCommandx"
 				|| t.cs() == "ProvideTextCommandDefault"
 				|| t.cs() == "DeclareMathAccent") {
 			bool star = false;
@@ -636,7 +848,7 @@ void parse_preamble(Parser & p, ostream & os,
 				star = true;
 			}
 			string const name = p.verbatim_item();
-			string const opt1 = p.getOpt();
+			string const opt1 = p.getFullOpt();
 			string const opt2 = p.getFullOpt();
 			string const body = p.verbatim_item();
 			// font settings
@@ -654,6 +866,10 @@ void parse_preamble(Parser & p, ostream & os,
 				// remove leading "\"
 				h_font_default_family = family.erase(0,1);
 			}
+
+			// Add the command to the known commands
+			add_known_command(name, opt1, !opt2.empty(), from_utf8(body));
+
 			// only non-lyxspecific stuff
 			if (!in_lyx_preamble) {
 				ostringstream ss;
@@ -663,9 +879,6 @@ void parse_preamble(Parser & p, ostream & os,
 				ss << '{' << name << '}' << opt1 << opt2
 				   << '{' << body << "}";
 				h_preamble << ss.str();
-
-				// Add the command to the known commands
-				add_known_command(name, opt1, !opt2.empty());
 /*
 				ostream & out = in_preamble ? h_preamble : os;
 				out << "\\" << t.cs() << "{" << name << "}"
@@ -688,7 +901,7 @@ void parse_preamble(Parser & p, ostream & os,
 			// options.
 			handle_opt(opts, known_languages, h_language);
 			delete_opt(opts, known_languages);
-			
+
 			// paper orientation
 			if ((it = find(opts.begin(), opts.end(), "landscape")) != opts.end()) {
 				h_paperorientation = "landscape";
@@ -723,6 +936,8 @@ void parse_preamble(Parser & p, ostream & os,
 			delete_opt(opts, known_class_paper_sizes);
 			// the remaining options
 			h_options = join(opts, ",");
+			// FIXME This does not work for classes that have a
+			//       different name in LyX than in LaTeX
 			h_textclass = p.getArg('{', '}');
 		}
 
@@ -746,21 +961,24 @@ void parse_preamble(Parser & p, ostream & os,
 
 		else if (t.cs() == "newenvironment") {
 			string const name = p.getArg('{', '}');
-			ostringstream ss;
-			// only non LyX specific stuff is output
-			ss << "\\newenvironment{" << name << "}";
-			ss << p.getOpt();
-			ss << p.getOpt();
-			ss << '{' << p.verbatim_item() << '}';
-			ss << '{' << p.verbatim_item() << '}';
-			if (!in_lyx_preamble)
-				h_preamble << ss.str();
+			string const opt1 = p.getFullOpt();
+			string const opt2 = p.getFullOpt();
+			string const beg = p.verbatim_item();
+			string const end = p.verbatim_item();
+			if (!in_lyx_preamble) {
+				h_preamble << "\\newenvironment{" << name
+				           << '}' << opt1 << opt2 << '{'
+				           << beg << "}{" << end << '}';
+			}
+			add_known_environment(name, opt1, !opt2.empty(),
+			                      from_utf8(beg), from_utf8(end));
+
 		}
 
 		else if (t.cs() == "def") {
 			string name = p.get_token().cs();
 			while (p.next_token().cat() != catBegin)
-				name += p.get_token().asString();
+				name += p.get_token().cs();
 			if (!in_lyx_preamble)
 				h_preamble << "\\def\\" << name << '{'
 					   << p.verbatim_item() << "}";
@@ -800,6 +1018,11 @@ void parse_preamble(Parser & p, ostream & os,
 			if (name == "\\parindent" && content != "") {
 				if (content[0] == '0')
 					h_paragraph_separation = "skip";
+				else if (LYX_FORMAT >= 365)
+					h_paragraph_indentation = translate_len(content);
+				else
+					h_preamble << "\\setlength{" << name
+					           << "}{" << content << "}";
 			} else if (name == "\\parskip") {
 				if (content == "\\smallskipamount")
 					h_defskip = "smallskip";
@@ -858,7 +1081,25 @@ void parse_preamble(Parser & p, ostream & os,
 			}
 		}
 
+		else if (t.cs() == "definecolor") {
+			string const color = p.getArg('{', '}');
+			string const space = p.getArg('{', '}');
+			string const value = p.getArg('{', '}');
+			if (LYX_FORMAT >= 382 &&
+			    color == "note_fontcolor" && space == "rgb") {
+				RGBColor c(RGBColorFromLaTeX(value));
+				h_notefontcolor = X11hexname(c);
+			} else {
+				h_preamble << "\\definecolor{" << color
+				           << "}{" << space << "}{" << value
+				           << '}';
+			}
+		}
+
 		else if (t.cs() == "jurabibsetup") {
+			// FIXME p.getArg('{', '}') is most probably wrong (it
+			//       does not handle nested braces).
+			//       Use p.verbatim_item() instead.
 			vector<string> jurabibsetup =
 				split_options(p.getArg('{', '}'));
 			// add jurabibsetup to the jurabib package options
@@ -869,6 +1110,40 @@ void parse_preamble(Parser & p, ostream & os,
 			}
 		}
 
+		else if (t.cs() == "hypersetup") {
+			vector<string> hypersetup =
+				split_options(p.verbatim_item());
+			// add hypersetup to the hyperref package options
+			handle_hyperref(hypersetup);
+			if (!hypersetup.empty()) {
+				h_preamble << "\\hypersetup{"
+				           << join(hypersetup, ",") << '}';
+			}
+		}
+
+		else if (is_known(t.cs(), known_if_3arg_commands)) {
+			// prevent misparsing of \usepackage if it is used
+			// as an argument (see e.g. our own output of
+			// \@ifundefined above)
+			string const arg1 = p.verbatim_item();
+			string const arg2 = p.verbatim_item();
+			string const arg3 = p.verbatim_item();
+			if (!in_lyx_preamble) {
+				h_preamble << t.asInput()
+				           << '{' << arg1 << '}'
+				           << '{' << arg2 << '}'
+				           << '{' << arg3 << '}';
+			}
+		}
+
+		else if (is_known(t.cs(), known_if_commands)) {
+			// must not parse anything in conditional code, since
+			// LyX would output the parsed contents unconditionally
+			if (!in_lyx_preamble)
+				h_preamble << t.asInput();
+			handle_if(p, in_lyx_preamble);
+		}
+
 		else if (!t.cs().empty() && !in_lyx_preamble)
 			h_preamble << '\\' << t.cs();
 	}
@@ -881,12 +1156,11 @@ void parse_preamble(Parser & p, ostream & os,
 		h_textclass = forceclass;
 	if (noweb_mode && !prefixIs(h_textclass, "literate-"))
 		h_textclass.insert(0, "literate-");
-	FileName layoutfilename = libFileSearch("layouts", h_textclass, "layout");
-	if (layoutfilename.empty()) {
-		cerr << "Error: Could not find layout file for textclass \"" << h_textclass << "\"." << endl;
-		exit(1);
+	tc.setName(h_textclass);
+	if (!tc.load()) {
+		cerr << "Error: Could not read layout file for textclass \"" << h_textclass << "\"." << endl;
+		exit(EXIT_FAILURE);
 	}
-	tc.read(layoutfilename);
 	if (h_papersides.empty()) {
 		ostringstream ss;
 		ss << tc.sides();
@@ -895,6 +1169,16 @@ void parse_preamble(Parser & p, ostream & os,
 	end_preamble(os, tc);
 }
 
+
+/// translates a babel language name to a LyX language name
+string babel2lyx(string const & language)
+{
+	char const * const * where = is_known(language, known_languages);
+	if (where)
+		return known_coded_languages[where - known_languages];
+	return language;
+}
+
 // }])
 
 
